/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import java.util.Collection;
import java.util.List;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.conqat.lib.commons.collections.Pair;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableLabelProviderBase;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.introspection.items.EObjectAware2IntrospectionDetailsItemBase;

/**
 * Base class for details UI implementations with filtered tree viewer.
 * 
 * @author hoelzl
 */
public abstract class EObjectAware2IntrospectionDetailsUIHandlerBase
		extends IntrospectionDetailsUIHandlerBase {

	/** Creates the default tree columns. */
	protected final void createDefaultTreeColumns(Tree tree, String text0, int width0, String text1,
			int width1, String text2, int width2, String text3, int width3) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText(text0);
		col0.setWidth(width0);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText(text1);
		col1.setWidth(width1);

		TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);
		col2.setText(text2);
		col2.setWidth(width2);

		TreeColumn col3 = new TreeColumn(tree, SWT.LEFT);
		col3.setText(text3);
		col3.setWidth(width3);
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new EObjectAware2TableLabelProvider();
	}

	/** {@inheritDoc} */
	@Override
	protected PatternFilter createPatternFilter() {
		PatternFilter pf = new TableViewerPatternFilter();
		pf.setIncludeLeadingWildcard(true);
		return pf;
	}

	/** Table label provider for handler registrations with two classes. */
	protected static class EObjectAware2TableLabelProvider extends TableLabelProviderBase {
		/** {@inheritDoc} */
		@Override
		public String getColumnText(Object element, int columnIndex) {
			if(element instanceof Class) {
				return columnIndex == 0 ? ((Class<?>)element).getSimpleName() : "";
			}
			if(element instanceof Pair) {
				Pair<?, ?> pair = (Pair<?, ?>)element;
				if(pair.getFirst() instanceof Class) {
					Class<?> c = (Class<?>)((Pair<?, ?>)element).getSecond();
					return columnIndex == 0 ? c.getSimpleName() : "";
				}
				if(pair.getFirst() != null) {
					Object provider = pair.getFirst();
					Pair<?, ?> regClasses = (Pair<?, ?>)pair.getSecond();
					Class<?> sourceClass = (Class<?>)regClasses.getFirst();
					Class<?> targetClass = (Class<?>)regClasses.getSecond();
					switch(columnIndex) {
						case 0:
							return provider.getClass().getSimpleName();
						case 1:
							return provider.getClass().getName();
						case 2:
							return sourceClass.getName();
						case 3:
							return targetClass.getName();
					}
				}
			}
			return "";
		}
	}

	/** Tree content provider for handler registrations with two classes. */
	protected static abstract class EObjectAware2TreeContentProviderBase
			extends TreeContentProviderBase {

		/** Returns the input object for this content provider. */
		protected abstract EObjectAware2IntrospectionDetailsItemBase<?> getInputObject();

		/** {@inheritDoc} */
		@Override
		public Object[] getChildren(Object parentElement) {
			if(parentElement == getInputObject()) {
				return getInputObject().getFirstHandlerKeyClasses().toArray();
			}
			if(parentElement instanceof Class<?>) {
				Class<?> firstClass = (Class<?>)parentElement;
				Collection<Class<?>> secondaryClasses =
						getInputObject().getSecondHandlerKeyClasses(firstClass);
				if(secondaryClasses == null || secondaryClasses.isEmpty()) {
					return new Object[0];
				}
				Pair<?, ?>[] pairs = new Pair<?, ?>[secondaryClasses.size()];
				int idx = 0;
				for(Class<?> secondary : secondaryClasses) {
					pairs[idx] = new Pair<Class<?>, Class<?>>(firstClass, secondary);
					idx++;
				}
				return pairs;
			}
			if(parentElement instanceof Pair) {
				Pair<?, ?> pair = (Pair<?, ?>)parentElement;
				if(pair.getFirst() instanceof Class) {
					Class<?> first = (Class<?>)pair.getFirst();
					Class<?> second = (Class<?>)pair.getSecond();
					List<?> handlerList = getInputObject().getHandlerList(first, second);
					if(handlerList == null || handlerList.isEmpty()) {
						return new Object[0];
					}
					Pair<?, ?>[] pairs = new Pair<?, ?>[handlerList.size()];
					int idx = 0;
					for(Object handler : handlerList) {
						pairs[idx] = new Pair<Object, Pair<?, ?>>(handler, pair);
						idx++;
					}
					return pairs;
				}
				// no children for handlers
			}
			return new Object[0];
		}
	}

	/** {@inheritDoc} */
	@Override
	protected boolean testSelection(Object selection) {
		if(selection instanceof Pair<?, ?>) {
			Pair<?, ?> pair = (Pair<?, ?>)selection;
			return !(pair.getFirst() instanceof Class<?>);
		}
		return false;
	}

	/** {@inheritDoc} */
	@Override
	protected void populateContextMenu(Object selection, MenuManager mgr) {
		Pair<?, ?> pair = (Pair<?, ?>)selection;
		mgr.add(createCopyClassNameAction(pair.getFirst().getClass()));
		Pair<?, ?> classPair = (Pair<?, ?>)pair.getSecond();
		mgr.add(createCopyClassNameAction((Class<?>)classPair.getFirst()));
		mgr.add(createCopyClassNameAction((Class<?>)classPair.getSecond()));
	}
}
