package org.fortiss.tooling.kernel.service;

import java.util.List;

import org.fortiss.tooling.kernel.extension.IConstraint;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.internal.ConstraintService;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.OutdatedConstraintInstanceStatus;

/**
 * The verification service allows to register constraints. See KISS documentation for more details.
 */
public interface IConstraintService {

	/** Returns the service instance. */
	public static IConstraintService getInstance() {
		return ConstraintService.getInstance();
	}

	/**
	 * Verifies the constraint instance <code>ci</code>. The service always runs the verification
	 * in a command. Should not yield any "useless" change because verification should always update
	 * the constraint instance status and the checksums anyways.
	 */
	void verify(ConstraintInstance ci);

	/**
	 * Tries to cancel the last verification of the constraint instance <code>ci</code>. The actual
	 * actions to take upon cancellation are up to the constraint.
	 */
	void cancel(ConstraintInstance ci);

	/**
	 * @return <code>true</code> if the constraint instance <code>ci</code> is up to date.
	 *         This does not just look at the status but computes a diff between the constrained
	 *         element and the one for which the constraint instance was last time checked. In the
	 *         standard case, it is enough to call IConstraintVerificationUIService.getStatus and
	 *         see if the result is an instance of {@link OutdatedConstraintInstanceStatus}. You
	 *         should use <code>isUpToDate</code> *only if
	 *         IConstraintVerificationUIService.getStatus does not provide the information you
	 *         want*.
	 */
	boolean isUpToDate(ConstraintInstance ci);

	/**
	 * Completes <code>top</code> with missing constraints instances of constraint handled by
	 * <code>cstrClass</code>, if applicable.
	 * 
	 * @return List of added constraints instances.
	 * 
	 */
	List<ConstraintInstance> addMissingConstraintsInstances(ITopLevelElement top,
			Class<? extends IConstraint> cstrClass);

	/** Returns the name of the given constraint, usable for serialization. */
	public String getName(Class<? extends IConstraint> cstrClass);

	/**
	 * @param cstrClass
	 * @return name of the group that a particular constraint belong to
	 */
	public String getGroupName(Class<? extends IConstraint> cstrClass);

	/** Returns a constraint, given its name. */
	public Class<? extends IConstraint> getConstraintByName(String name);

	/** Registers the given verifier with the service. */
	void registerConstraint(Class<? extends IConstraint> cstrClass);

	/** <code>true</code> if the given instance is queued for update. */
	boolean isUpdating(ConstraintInstance ci);

	/**
	 * Marks the given instance as being queued for update.
	 * 
	 * ONLY FOR INTERNAL USE.
	 */
	void markAsUpdating(ConstraintInstance ci);
}
