/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.selection;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;

/**
 * This base class is used for writing selection listeners which attempt to set
 * their own selection according to the externally observed selection. Such a
 * mechanism can be used for example to keep selections in an editor and the
 * outline view the same. The tricky part is to not fire a selection changed
 * event when the selection is changed not due to use input but rather because
 * of an external change event.
 * <p>
 * To use this class, your selectionChanged or fireSelectionChanged method
 * should be changed as this:
 * 
 * <pre>
 * SynchronizingSelectionListener ssl = new SynchronizingSelectionListener() {...};
 * // ...
 * public void selectionChanged(SelectionChangedEvent event) {
 * 	synchronized (ssl) {
 * 		if (!ssl.isExternalChange()) {
 * 			super.selectionChanged(event);
 * 		}
 * 	}
 * }
 * </pre>
 * <p>
 * This only works for structured selections.
 * 
 * @author hummelb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: EEF2F6EC3895CA753467C6D26536E680
 */
public abstract class SynchronizingSelectionListenerBase implements
		ISelectionListener {

	/**
	 * Flag indicating that a selection change was external. Access to this flag
	 * is protected by synchronization to this object.
	 */
	private boolean externalSelection = false;

	/** {@inheritDoc} */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		synchronized (this) {
			externalSelection = true;
			try {
				if (selection instanceof IStructuredSelection) {
					List<Object> elements = new ArrayList<Object>();
					for (Object o : ((IStructuredSelection) selection).toList()) {
						Object converted = convertObject(o);
						if (converted != null) {
							elements.add(converted);
						}
					}
					setSelection(new StructuredSelection(elements));
				}
			} finally {
				externalSelection = false;
			}
		}
	}

	/**
	 * Template method which allows to change the objects before adding them to
	 * the internal selection. This can be used to wrap or unwrap objects (e.g.
	 * edit parts for GEF).
	 */
	protected Object convertObject(Object o) {
		return o;
	}

	/**
	 * This method performs the actual setting of the selection to the local
	 * view or editor.
	 */
	protected abstract void setSelection(StructuredSelection selection);

	/**
	 * Returns if the current selection change event is caused by an external
	 * selection and thus should not be propagated. Access should be
	 * synchronized to this object.
	 */
	public synchronized boolean isExternalSelection() {
		return externalSelection;
	}

}